/******************************************************************************
*
*      Authors:  K. Chaudhary / S. Nag / H. Varma
*      Company:  Xilinx
*
*   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
*                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
*                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
*                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
*
*                Copyright (c) 1999 Xilinx, Inc.
*                All rights reserved
*
*
******************************************************************************/

`timescale 1ns / 1ps

// LUT definition for instantiation
module LUT4(O, I0, I1, I2, I3); // synthesis xc_map=lut
output O;
input I0;
input I1;
input I2;
input I3;
parameter INIT = 16'b0;
assign O = I3 ?
        (I2 ?
         (I1 ? (I0 ? INIT[15] : INIT[14]) : (I0 ? INIT[13] : INIT[12])) : 
         (I1 ? (I0 ? INIT[11] : INIT[10]) : (I0 ? INIT[9] : INIT[8]))) :
        (I2 ?
         (I1 ? (I0 ? INIT[7] : INIT[6]) : (I0 ? INIT[5] : INIT[4])) : 
         (I1 ? (I0 ? INIT[3] : INIT[2]) : (I0 ? INIT[1] : INIT[0])));
endmodule

/*****************************************************************************
*
* Constants in 2D IDCT follow following conventions:
*    Ck  = cos(k*pi/16) when k>0
*        = sqrt(0.5)    when k==0
*
*
*
* These are the init values for the LUTs used in bug luts
*
*
* FOR C0 C2 C4 C6 ==> biglut0
*     defparam L0.INIT = 'hA424;
*     defparam L1.INIT = 'h9292;
*     defparam L2.INIT = 'h817E;
*     defparam L3.INIT = 'h25A4;
*     defparam L4.INIT = 'hA05E;
*     defparam L5.INIT = 'h3668;
*     defparam L6.INIT = 'hC880;
*     defparam L7.INIT = 'h0000;
* FOR C1 C3 C5 C7 ==> biglut1
*     defparam L0.INIT = 'h22BA;
*     defparam L1.INIT = 'hE186;
*     defparam L2.INIT = 'h2342;
*     defparam L3.INIT = 'h10CE;
*     defparam L4.INIT = 'hF03E;
*     defparam L5.INIT = 'h6668;
*     defparam L6.INIT = 'h8880;
*     defparam L7.INIT = 'h0000;
* FOR C0 C6 -C4 -C2 ==> biglut2
*     defparam L0.INIT = 'hED00;
*     defparam L1.INIT = 'hA75A;
*     defparam L2.INIT = 'h9CC6;
*     defparam L3.INIT = 'hD694;
*     defparam L4.INIT = 'h4242;
*     defparam L5.INIT = 'hA758;
*     defparam L6.INIT = 'hF750;
*     defparam L7.INIT = 'hF750;
* FOR C3 -C7 -C1 -C5 ==> biglut3
*     defparam L0.INIT = 'h51D0;
*     defparam L1.INIT = 'h3946;
*     defparam L2.INIT = 'h91A8;
*     defparam L3.INIT = 'h7766;
*     defparam L4.INIT = 'h00EE;
*     defparam L5.INIT = 'hA5B4;
*     defparam L6.INIT = 'hF5F4;
*     defparam L7.INIT = 'hF5F4;
* FOR C0 -C6 -C4 C2 ==> biglut4
*     defparam L0.INIT = 'hED00;
*     defparam L1.INIT = 'h585A;
*     defparam L2.INIT = 'h3BC6;
*     defparam L3.INIT = 'h6158;
*     defparam L4.INIT = 'hE7C6;
*     defparam L5.INIT = 'h4A94;
*     defparam L6.INIT = 'h40D4;
*     defparam L7.INIT = 'h40D4;
* FOR C5 -C1 C7 C3 ==> biglut5
*     defparam L0.INIT = 'hC8CE;
*     defparam L1.INIT = 'hA158;
*     defparam L2.INIT = 'h0E50;
*     defparam L3.INIT = 'h0FA0;
*     defparam L4.INIT = 'hA5AA;
*     defparam L5.INIT = 'h36CC;
*     defparam L6.INIT = 'h04CC;
*     defparam L7.INIT = 'h04CC;
* FOR C0 -C2 C4 -C6 ==> biglut6
*     defparam L0.INIT = 'hECA4;
*     defparam L1.INIT = 'h96DE;
*     defparam L2.INIT = 'h4DB2;
*     defparam L3.INIT = 'h5AE8;
*     defparam L4.INIT = 'h055A;
*     defparam L5.INIT = 'h696C;
*     defparam L6.INIT = 'h4D4C;
*     defparam L7.INIT = 'h4D4C;
* FOR C7 -C5 C3 -C1 ==> biglut7
*     defparam L0.INIT = 'h278C;
*     defparam L1.INIT = 'h4E96;
*     defparam L2.INIT = 'h1E8E;
*     defparam L3.INIT = 'h54D4;
*     defparam L4.INIT = 'h9818;
*     defparam L5.INIT = 'hD32C;
*     defparam L6.INIT = 'hDF0C;
*     defparam L7.INIT = 'hDF0C;
*
* In the following Big LUTs ==>
*    dout[7:0] dout[7] is sign bit. dout[6] is MSB. dout[0] is LSB.
*      decimal position is after dout[5]
*    din3 is the MSB address in the Big LUTs
*    din0 is the LSB address of the LUT
*
******************************************************************************/


module biglut0 (dout, din0, din1, din2, din3);
output [7:0] dout;
input din0;
input din1;
input din2;
input din3;

  LUT4 L0 (.O(dout[0]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L0.INIT = 'hA424;
  LUT4 L1 (.O(dout[1]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L1.INIT = 'h9292;
  LUT4 L2 (.O(dout[2]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L2.INIT = 'h817E;
  LUT4 L3 (.O(dout[3]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L3.INIT = 'h25A4;
  LUT4 L4 (.O(dout[4]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L4.INIT = 'hA05E;
  LUT4 L5 (.O(dout[5]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L5.INIT = 'h3668;
  LUT4 L6 (.O(dout[6]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L6.INIT = 'hC880;
  LUT4 L7 (.O(dout[7]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L7.INIT = 'h0000;

endmodule

module biglut1(dout, din0, din1, din2, din3);

input din0;
input din1;
input din2;
input din3;
output [7:0] dout;

  LUT4 L0 (.O(dout[0]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L0.INIT = 'h22BA;
  LUT4 L1 (.O(dout[1]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L1.INIT = 'hE186;
  LUT4 L2 (.O(dout[2]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L2.INIT = 'h2342;
  LUT4 L3 (.O(dout[3]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L3.INIT = 'h10CE;
  LUT4 L4 (.O(dout[4]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L4.INIT = 'hF03E;
  LUT4 L5 (.O(dout[5]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L5.INIT = 'h6668;
  LUT4 L6 (.O(dout[6]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L6.INIT = 'h8880;
  LUT4 L7 (.O(dout[7]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L7.INIT = 'h0000;

endmodule

module biglut2(dout, din0, din1, din2, din3);

input din0;
input din1;
input din2;
input din3;
output [7:0] dout;

  LUT4 L0 (.O(dout[0]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L0.INIT = 'hED00;
  LUT4 L1 (.O(dout[1]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L1.INIT = 'hA75A;
  LUT4 L2 (.O(dout[2]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L2.INIT = 'h9CC6;
  LUT4 L3 (.O(dout[3]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L3.INIT = 'hD694;
  LUT4 L4 (.O(dout[4]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L4.INIT = 'h4242;
  LUT4 L5 (.O(dout[5]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L5.INIT = 'hA758;
  LUT4 L6 (.O(dout[6]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L6.INIT = 'hF750;
  LUT4 L7 (.O(dout[7]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L7.INIT = 'hF750;

endmodule

module biglut3(dout, din0, din1, din2, din3);

input din0;
input din1;
input din2;
input din3;
output [7:0] dout;

  LUT4 L0 (.O(dout[0]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L0.INIT = 'h51D0;
  LUT4 L1 (.O(dout[1]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L1.INIT = 'h3946;
  LUT4 L2 (.O(dout[2]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L2.INIT = 'h91A8;
  LUT4 L3 (.O(dout[3]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L3.INIT = 'h7766;
  LUT4 L4 (.O(dout[4]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L4.INIT = 'h00EE;
  LUT4 L5 (.O(dout[5]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L5.INIT = 'hA5B4;
  LUT4 L6 (.O(dout[6]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L6.INIT = 'hF5F4;
  LUT4 L7 (.O(dout[7]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L7.INIT = 'hF5F4;

endmodule

module biglut4(dout, din0, din1, din2, din3);

input din0;
input din1;
input din2;
input din3;
output [7:0] dout;

  LUT4 L0 (.O(dout[0]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L0.INIT = 'hED00;
  LUT4 L1 (.O(dout[1]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L1.INIT = 'h585A;
  LUT4 L2 (.O(dout[2]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L2.INIT = 'h3BC6;
  LUT4 L3 (.O(dout[3]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L3.INIT = 'h6158;
  LUT4 L4 (.O(dout[4]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L4.INIT = 'hE7C6;
  LUT4 L5 (.O(dout[5]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L5.INIT = 'h4A94;
  LUT4 L6 (.O(dout[6]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L6.INIT = 'h40D4;
  LUT4 L7 (.O(dout[7]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L7.INIT = 'h40D4;

endmodule

module biglut5(dout, din0, din1, din2, din3);

input din0;
input din1;
input din2;
input din3;
output [7:0] dout;

  LUT4 L0 (.O(dout[0]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L0.INIT = 'hC8CE;
  LUT4 L1 (.O(dout[1]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L1.INIT = 'hA158;
  LUT4 L2 (.O(dout[2]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L2.INIT = 'h0E50;
  LUT4 L3 (.O(dout[3]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L3.INIT = 'h0FA0;
  LUT4 L4 (.O(dout[4]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L4.INIT = 'hA5AA;
  LUT4 L5 (.O(dout[5]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L5.INIT = 'h36CC;
  LUT4 L6 (.O(dout[6]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L6.INIT = 'h04CC;
  LUT4 L7 (.O(dout[7]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L7.INIT = 'h04CC;

endmodule

module biglut6(dout, din0, din1, din2, din3);

input din0;
input din1;
input din2;
input din3;
output [7:0] dout;

  LUT4 L0 (.O(dout[0]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L0.INIT = 'hECA4;
  LUT4 L1 (.O(dout[1]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L1.INIT = 'h96DE;
  LUT4 L2 (.O(dout[2]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L2.INIT = 'h4DB2;
  LUT4 L3 (.O(dout[3]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L3.INIT = 'h5AE8;
  LUT4 L4 (.O(dout[4]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L4.INIT = 'h055A;
  LUT4 L5 (.O(dout[5]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L5.INIT = 'h696C;
  LUT4 L6 (.O(dout[6]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L6.INIT = 'h4D4C;
  LUT4 L7 (.O(dout[7]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L7.INIT = 'h4D4C;

endmodule

module biglut7(dout, din0, din1, din2, din3);

input din0;
input din1;
input din2;
input din3;
output [7:0] dout;

  LUT4 L0 (.O(dout[0]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L0.INIT = 'h278C;
  LUT4 L1 (.O(dout[1]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L1.INIT = 'h4E96;
  LUT4 L2 (.O(dout[2]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L2.INIT = 'h1E8E;
  LUT4 L3 (.O(dout[3]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L3.INIT = 'h54D4;
  LUT4 L4 (.O(dout[4]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L4.INIT = 'h9818;
  LUT4 L5 (.O(dout[5]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L5.INIT = 'hD32C;
  LUT4 L6 (.O(dout[6]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L6.INIT = 'hDF0C;
  LUT4  L7 (.O(dout[7]), .I0(din0), .I1(din1), .I2(din2), .I3(din3));
    defparam L7.INIT = 'hDF0C;

endmodule
