/******************************************************************************
*
*      Authors:  K. Chaudhary / S. Nag / H. Varma
*      Company:  Xilinx
*
*   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
*                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
*                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
*                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
*
*                Copyright (c) 1999 Xilinx, Inc.
*                All rights reserved
*
******************************************************************************/

`timescale 1ns / 1ps

`define C0 0.7071067811865476
`define C1 0.9807852804032304
`define C2 0.9238795325112867
`define C3 0.8314696123025452
`define C4 0.7071067811865476
`define C5 0.5555702330196023
`define C6 0.3826834323650898
`define C7 0.1950903220161283

`define CR0 0000741455
`define CR1 0001028427
`define CR2 0000968757
`define CR3 0000871859
`define CR4 0000741455
`define CR5 0000582557
`define CR6 0000401272
`define CR7 0000204567

`define CLK_PER 100
`define HALF_CLK_PER 50

// sample test bench

module tb_idct;

reg [0:767] din;
reg clk;
reg go;
reg rst;

wire done;
wire [0:767] dout;

reg [0:11] rbv;

idct IDCT1 (din, dout, clk, go, rst, done);

  function real cos;
	input [3:0] x;
	input [3:0] mu;
	integer k;
     begin
	k = (2*x+1)*mu;
	k = k % 32;
	if (k >= 16)
		k = 32 - k;
	case (k) 
	0:	cos = `C0;
	1:	cos = `C1;
	2:	cos = `C2;
	3:	cos = `C3;
	4:	cos = `C4;
	5:	cos = `C5;
	6:	cos = `C6;
	7:	cos = `C7;
	8:	cos = 0;
	9:	cos = -`C7;
	10: cos = -`C6;
	11: cos = -`C5;
	12: cos = -`C4;
	13: cos = -`C3;
	14: cos = -`C2;
	15: cos = -`C1;
	16: cos = 0;
	endcase
     end
  endfunction	

function real IDCT;
	input [767:0] din;
	input [3:0] x;
	input [3:0] y;
	integer i, j, m;
	integer dres;
	begin
	IDCT = 0.0;
  	for (i=0; i<8; i=i+1) begin
    		for (j=0; j<8; j=j+1) begin
      			for (m=0; m<12; m=m+1)
        			rbv[m] = din[767-(96*i+12*j+m)];
      			dres = rbv;
      			if (dres >= 2048) 
        			dres = dres-4096;      
			IDCT = IDCT + cos(x, i)*cos(y, j)*dres;			
		end
	end
	IDCT = IDCT/4.0;
	end
endfunction

integer i, j, m;
integer dres;
real expected;

always
  #`HALF_CLK_PER clk = ~clk;

initial
begin

  din =  {12'd256, 84'b0,
          12'd120, 84'b0,
	  12'd20, 84'b0,
          12'd10, 84'b0,
          84'b0, 12'd5,
          12'd10, 84'b0,
          84'b0, 12'd0,
          12'd0, 84'b0}; 
	
  clk = 1'b0;
  rst = 1'b1;
  #`CLK_PER;

  rst = 1'b0;
  go = 1'b1;
  #`CLK_PER;

  go = 1'b0;
  i = 0;
  while (done != 1'b1) 
  begin
    #`CLK_PER;
    i = i+1;
  end

  $display("Cycles Taken = %d", i);

  for (i=0; i<8; i=i+1) begin
    for (j=0; j<8; j=j+1) begin
      expected =  IDCT(din, i, j);
      for (m=0; m<12; m=m+1)
        rbv[m] = dout[96*i+12*j+m];
      dres = rbv;
      if (dres >= 2048) 
        dres = dres-4096;      
      $display("Row %d Col %d :  Result = %d Expected = %f", i, j, dres, expected);
    end
  end
  $display("Done 1");

  din =   {12'd3475, 12'd3042, 12'd19, 60'b0,
          12'd39, 12'd23, 72'b0,
	  12'd21, 84'b0,
          12'd0, 84'b0,
          84'b0, 12'd0,
          12'd0, 84'b0,
          84'b0, 12'd0,
          12'd0, 84'b0}; 

  #`CLK_PER;
  go = 1'b1;
  #`CLK_PER;
  go = 1'b0;

  i = 0;
  while (done != 1'b1) 
  begin
    #`CLK_PER;
    i = i+1;
  end

  $display("Cycles Taken = %d", i);

  for (i=0; i<8; i=i+1) begin
    for (j=0; j<8; j=j+1) begin
      expected =  IDCT(din, i, j);
      for (m=0; m<12; m=m+1)
        rbv[m] = dout[96*i+12*j+m];
      dres = rbv;
      if (dres >= 2048) 
        dres = dres-4096;      
      $display("Row %d Col %d :  Result = %d Expected = %f", i, j, dres, expected);
    end
  end
  $display("Done 2");
// $finish;
end


endmodule